"""This module contains the ``exporters`` related test cases"""

from unittest import TestCase
from unittest.mock import call, patch

from scrapy_algolia_exporter.exporters import AlgoliaItemExporter


class AlgoliaItemExporterTestCase(TestCase):
    """Test case for the ``AlgoliaItemExporter``"""

    def setUp(self):
        """Initialize the ``AlgoliaItemExporter``"""

        self.algolia_item_exporter = AlgoliaItemExporter(
            algolia_api_id='api_id',
            algolia_api_key='api_key',
            algolia_index_name='index_name'
        )

    def test_start_exporting_should_initialize_the_algolia_client(self):
        """Test that the ``start_exporting`` method should initialize the Algolia client"""

        with patch('algoliasearch.algoliasearch.Client') as mocked_algolia_client:
            self.algolia_item_exporter.start_exporting()

        mocked_algolia_client.assert_has_calls(
            [
                call('api_id', 'api_key'),
                call().init_index('index_name')
            ]
        )
        self.assertEqual(self.algolia_item_exporter.next_items, [])
        self.assertEqual(self.algolia_item_exporter.exported_items_nbr, 0)

    def test_export_item_should_add_the_items_to_the_stack(self):
        """Test that the ``export_item`` method should add the item to the queue"""

        with patch('algoliasearch.algoliasearch.Client'):
            self.algolia_item_exporter.start_exporting()

        # At first, we don't have any items in the queue
        self.assertEqual(self.algolia_item_exporter.next_items, [])

        # Export an item
        self.algolia_item_exporter.export_item({'id': 1})

        # We should have a new item in the queue
        self.assertEqual(self.algolia_item_exporter.next_items, [{'id': 1}])

    def test_export_item_should_call__export_items_if_algolia_item_bulk_nbr_is_reached(self):
        """Test that the ``export_item`` method should call ``_export_items`` if ``algolia_item_bulk_nbr`` is reached"""

        with patch('algoliasearch.algoliasearch.Client'):
            self.algolia_item_exporter.start_exporting()

        # Add 100 items
        items = [
            {'id': i}
            for i in range(0, 200)
        ]
        with patch('scrapy_algolia_exporter.exporters.AlgoliaItemExporter._export_items') as mocked:
            for item in items:
                self.algolia_item_exporter.export_item(item)

        # The ``_export_items`` was called only once
        mocked.assert_called_once_with(items)



"""This module contains the ``pipelines`` related test cases"""

from unittest import TestCase
from unittest.mock import Mock

from scrapy.exceptions import NotConfigured
from scrapy.crawler import Crawler

from scrapy_algolia_exporter.pipelines import AlgoliaItemPipeline


class AlgoliaItemPipelineTestCase(TestCase):
    """Test case for the ``AlgoliaItemPipeline``"""

    def setUp(self):
        """Create a fake scrapy ``Crawler``"""

        self.crawler = Crawler(
            spidercls=Mock(),
            settings={
                'ALGOLIA_API_ID': 'api_id',
                'ALGOLIA_API_KEY': 'api_key',
                'ALGOLIA_INDEX_NAME': 'index_name'
            }
        )

    def test_from_crawler_should_initialize_the_pipeline(self):
        """Test that the ``from_crawler`` method should correctly initialize the pipeline"""

        pipeline = AlgoliaItemPipeline.from_crawler(crawler=self.crawler)

        self.assertEqual(pipeline.algolia_api_id, 'api_id')
        self.assertEqual(pipeline.algolia_api_key, 'api_key')
        self.assertEqual(pipeline.algolia_index_name, 'index_name')

    def test_missing_mandatory_settings_should_raise_an_exception(self):
        """Test that the ``from_crawler`` method should correctly initialize the pipeline"""

        with self.assertRaises(NotConfigured):
            AlgoliaItemPipeline.from_crawler(
                crawler=Crawler(
                    spidercls=Mock(),
                    settings={
                        'ALGOLIA_API_KEY': 'api_key',
                        'ALGOLIA_INDEX_NAME': 'index_name'
                    }
                )
            )



"""This module contains the ``AlgoliaItemPipeline``"""

from scrapy.exceptions import NotConfigured

from .exporters import AlgoliaItemExporter


class AlgoliaItemPipeline:
    """Export the scraped items to the Algolia API"""

    exporter = None

    def __init__(self,
                 algolia_api_id,
                 algolia_api_key,
                 algolia_index_name,
                 algolia_item_bulk_nbr):
        """Store the Algolia settings"""

        self.algolia_api_id = algolia_api_id
        self.algolia_api_key = algolia_api_key
        self.algolia_index_name = algolia_index_name
        self.algolia_item_bulk_nbr = algolia_item_bulk_nbr

    @classmethod
    def from_crawler(cls, crawler):
        """Get the settings and initialize"""

        algolia_api_id = crawler.settings['ALGOLIA_API_ID']
        algolia_api_key = crawler.settings['ALGOLIA_API_KEY']
        algolia_index_name = crawler.settings['ALGOLIA_INDEX_NAME']

        if not algolia_api_id or not algolia_api_key or not algolia_index_name:
            raise NotConfigured(
                'Missing configuration for the AlgoliaItemPipeline'
            )

        algolia_item_bulk_nbr = crawler.settings.get(
            'ALGOLIA_ITEM_BULK_NBR',
            100
        )

        return cls(
            algolia_api_id,
            algolia_api_key,
            algolia_index_name,
            algolia_item_bulk_nbr
        )

    def open_spider(self, spider):
        """Initialize the ``AlgoliaItemExporter``"""

        self.exporter = AlgoliaItemExporter(
            algolia_api_id=self.algolia_api_id,
            algolia_api_key=self.algolia_api_key,
            algolia_index_name=self.algolia_index_name,
            algolia_item_bulk_nbr=self.algolia_item_bulk_nbr
        )
        self.exporter.start_exporting()

    def close_spider(self, spider):
        self.exporter.finish_exporting()

    def process_item(self, item, spider):
        """Export the item using the ``AlgoliaItemExporter``"""

        self.exporter.export_item(item)

        return item



"""This module contains the ``AlgoliaItemExporter``"""

from algoliasearch import algoliasearch


class AlgoliaItemExporter:
    """Export the scraped items to the Algolia API"""

    algolia_index = None
    next_items = None
    exported_items_nbr = None

    def __init__(self,
                 algolia_api_id,
                 algolia_api_key,
                 algolia_index_name,
                 algolia_item_bulk_nbr=100):

        self.algolia_api_id = algolia_api_id
        self.algolia_api_key = algolia_api_key
        self.algolia_index_name = algolia_index_name
        self.algolia_item_bulk_nbr = algolia_item_bulk_nbr

    def start_exporting(self):
        """Initialize the Algolia API Client"""

        client = algoliasearch.Client(
            self.algolia_api_id,
            self.algolia_api_key
        )
        self.algolia_index = client.init_index(self.algolia_index_name)

        self.next_items = []
        self.exported_items_nbr = 0

    def _export_items(self, items):
        """Export the given items to the current Algolia index"""

        self.algolia_index.add_objects(items)

    def finish_exporting(self):
        """Export the last items from the queue, if any"""

        if self.next_items:
            self._export_items(self.next_items)

    def export_item(self, item):
        """Add the given item to the queue, if the queue if full, upload the items"""

        self.next_items.append(item)

        if len(self.next_items) == self.algolia_item_bulk_nbr:
            self._export_items(self.next_items)







"""This module contains the packaging routine for the ``scrapy-algolia-exporter`` package"""

from setuptools import setup, find_packages
from pip.download import PipSession
from pip.req import parse_requirements


def get_requirements(source):
    """Get the requirements from the given ``source``

    Parameters
    ----------
    source: str
        The filename containing the requirements

    """

    install_reqs = parse_requirements(filename=source, session=PipSession())

    return [str(ir.req) for ir in install_reqs]

setup(
    packages=find_packages(),
    install_requires=get_requirements('requirements/requirements.txt')
)


